'\"
'\" Generated from file 'parse.man' by tcllib/doctools with format 'nroff'
'\"
'\" -*- tcl -*- doctools manpage
'\" $Id$
.so man.macros
.TH "parse" n 1.4  "Parse a Tcl script into commands, words, and tokens"
.BS
.SH "NAME"
parse \- System to control logging of events.
.SH "SYNOPSIS"
package require \fBTcl  8\fR
.sp
package require \fBparser  ?1.4?\fR
.sp
\fBparse\fR command \fIscript\fR [arg first] [arg last]\fR
.sp
\fBparse\fR expr \fIscript\fR [arg first] [arg last]\fR
.sp
\fBparse\fR varname \fIscript\fR [arg first] [arg last]\fR
.sp
\fBparse\fR subcommand \fIscript\fR \fIfirst\fR \fIlast\fR\fR
.sp
\fBparse\fR list \fIscript\fR [arg first] [arg last]\fR
.sp
\fBparse\fR getrange \fIstring\fR ?index length?\fR
.sp
\fBparse\fR getstring \fIstring\fR [arg first] [arg last]\fR
.sp
\fBparse\fR charindex \fIstring\fR [arg first] [arg last]\fR
.sp
\fBparse\fR charlength \fIstring\fR [arg first] [arg last]\fR
.sp
.BE
.SH "DESCRIPTION"
.PP
This command parses a Tcl script into \fIcommands, words\fR and \fItokens\fR.
Each of the commands below takes a \fIscript\fR to parse and a range
into the script: {\fIfirst\fR \fIlast\fR}.  The command parses the script from
the first index up to and including the \fIlast\fR index.  The return of
each command is a list of tuples indicating the ranges of each
sub-element.  Use the returned indices as arguments to \fBparse getstring\fR to
extract the parsed string from the script.
.PP
The \fBparse\fR command breaks up the script into sequentially smaller
elements.  A \fIscript\fR contains one or more \fIcommands\fR.  A \fIcommand\fR is a set
of \fIwords\fR that is terminated by a semicolon, newline or end the of the
script and has no unclosed quotes, braces, brackets or array element
names.  A \fIword\fR is a set of characters grouped together by whitespace,
quotes, braces or brackets.  Each word is composed of one or more
\fItokens\fR.  A \fItoken\fR is one of the following types: \fItext\fR, \fIvariable\fR,
\fIbackslash\fR, \fIcommand\fR, \fIexpr\fR, or \fIoperator\fR.  The type of token
specifies how to decompose the string further.  For example, a \fItext\fR
token is a literal set of characters that does not need to be broken
into smaller pieces.  However, the \fIvariable\fR token needs to be broken
into smaller pieces to separate the name of the variable from an array
indices, if one is supplied.
.PP
The \fIfirst\fR and \fIlast\fR indices are treated the same way as the indices in
the Tcl \fBstring\fR command.  An index of 0 refers to the first character
of the string.  An index of end (or any abbreviation of it) refers to
the last character of the string.  If first is less than zero then it
is treated as if it were zero, and if last is greater than or equal to
the length of the string then it is treated as if it were end.  If
first is greater than last then an empty string is returned.
.TP
\fBparse\fR command \fIscript\fR [arg first] [arg last]\fR
Returns a list of indices that partitions the script into \fIcommands\fR.
This routine returns a list of the following form: \fIcommentRange\fR
\fIcommandRange\fR \fIrestRange\fR \fIparseTree\fR. The first range refers to any
leading comments before the command.  The second range refers to the
command itself.  The third range contains the remainder of the
original range that appears after the command range.  The \fIparseTree\fR is
a list representation of the parse tree where each node is a list in
the form: \fItype\fR \fIrange\fR \fIsubTree\fR.
.TP
\fBparse\fR expr \fIscript\fR [arg first] [arg last]\fR
Returns a list that partitions an \fIexpression\fR into
subexpressions.  The first element of the list is the token type,
\fIsubexpr\fR, followed by the range of the expressions text, and
finally by a \fIsubTree\fR with the words and types of the parse
tree.
.TP
\fBparse\fR varname \fIscript\fR [arg first] [arg last]\fR
Returns a list that partitions a \fIvariable\fR token into words.
The first element of the list is the token type, \fIvariable\fR.  The
second is the range of the variable's text, and the third is a subTree
that lists the words and ranges of the variable's components.
.TP
\fBparse\fR subcommand \fIscript\fR \fIfirst\fR \fIlast\fR\fR
Returns a list of indices that partitions a subcommand token into
words. The first element in the list is the beginning index of the
subcommand token, and the second element in the list is the end index
of the subcommand token. The pattern is repeated for each subcommand
token in the script.  The first index must point to the character '['
and the last index must point to the ']' character otherwise the
command will return an empty string.  Each of the partitioned strings
should be parsed further by using the parse token command.
.TP
\fBparse\fR list \fIscript\fR [arg first] [arg last]\fR
Parses a script as a \fIlist\fR, returning the range of each element.
\fIscript\fR must be a valid list, or an error will be generated.
.TP
\fBparse\fR getrange \fIstring\fR ?index length?\fR
Gets the range in bytes of \fIstring\fR, optionally beginning at ?index?
of length ?length? (both in characters).  Equivalent to \fBstring bytelength\fR.
.TP
\fBparse\fR getstring \fIstring\fR [arg first] [arg last]\fR
Get the section of \fIstring\fR that corresponds to the specified
range (in bytes).  Note that this command must be used instead of \fBstring range\fR
with values returned from the parse commands, because the values are
in bytes, and \fBstring range\fR instead uses characters as its units.
.TP
\fBparse\fR charindex \fIstring\fR [arg first] [arg last]\fR
Converts byte oriented index values into character oriented index
values, for the string in question.
.TP
\fBparse\fR charlength \fIstring\fR [arg first] [arg last]\fR
Converts the given byte length into a character count, for the string in question.
.SH "EXAMPLES"
.nf
set script {
    while true {puts [getupdate]}
}

parse command $script {0 end}
.fi
Returns:
.PP
{0 0} {5 30} {35 0} {{simple {5 5} {{text {5 5} {}}}} {simple {11 4} {{text {11 4} {}}}} {simple {16 18} {{text {17 16} {}}}}}
.PP
Or in other words, a string with no comments, 30 bytes long, beginning
at byte 5.  It is composed of a series of subwords, which include
while, true, and {puts [getupdate]}.
.SH "KEYWORDS"
parse, parser